@@ -37,6 +37,7 @@ ASTERISK_REGISTER_FILE()
 #include <pjlib.h>
 #endif
 
+#include "asterisk/options.h"
 #include "asterisk/_private.h" /* ast_pj_init() */
 
 /*!
@@ -46,6 +47,7 @@ ASTERISK_REGISTER_FILE()
 int ast_pj_init(void)
 {
 #ifdef HAVE_PJPROJECT_BUNDLED
+	AST_PJPROJECT_INIT_LOG_LEVEL();
 	pj_init();
 #endif
 	return 0;
