@@ -1453,7 +1453,9 @@ int ast_careful_fwrite(FILE *f, int fd, const char *src, size_t len, int timeout
 
 		if (ferror(f) && errno != EINTR && errno != EAGAIN) {
 			/* fatal error from fwrite() */
-			if (!feof(f)) {
+			if (errno == EPIPE) {
+				ast_debug(1, "fwrite() failed due to reading end being closed: EPIPE\n");
+			} else if (!feof(f)) {
 				/* Don't spam the logs if it was just that the connection is closed. */
 				ast_log(LOG_ERROR, "fwrite() returned error: %s\n", strerror(errno));
 			}
@@ -1486,8 +1488,12 @@ int ast_careful_fwrite(FILE *f, int fd, const char *src, size_t len, int timeout
 			continue;
 		}
 		if (errno && !feof(f)) {
-			/* Don't spam the logs if it was just that the connection is closed. */
-			ast_log(LOG_ERROR, "fflush() returned error: %s\n", strerror(errno));
+			if (errno == EPIPE) {
+				ast_debug(1, "fflush() failed due to reading end being closed: EPIPE\n");
+			} else {
+				/* Don't spam the logs if it was just that the connection is closed. */
+				ast_log(LOG_ERROR, "fflush() returned error: %s\n", strerror(errno));
+			}
 		}
 		n = -1;
 		break;
@@ -2425,6 +2431,18 @@ char *ast_utils_which(const char *binary, char *fullpath, size_t fullpath_size)
 	return NULL;
 }
 
+int ast_check_ipv6(void)
+{
+	int udp6_socket = socket(AF_INET6, SOCK_DGRAM, 0);
+
+	if (udp6_socket < 0) {
+		return 0;
+	}
+
+	close(udp6_socket);
+	return 1;
+}
+
 void DO_CRASH_NORETURN ast_do_crash(void)
 {
 #if defined(DO_CRASH)
